package cn.dankal.share.common;

/**
 * 插入排序简单实现
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-02
 */
public class InsertionSort {

    /**
     * 排序方式
     */
    public enum Sort {
        /**
         * 倒序排序
         */
        DESC,
        /**
         * 顺序排序
         */
        ASC
    }

    /**
     * 使用插入排序顺序排序目标数组
     *
     * @param unSortedArray 未排序的数组
     * @return int[]  已经排序的数组
     */
    public static int[] doAscSort(int[] unSortedArray) {
        return doSort(unSortedArray, Sort.ASC);
    }

    /**
     * 使用插入排序倒序排序目标数组
     *
     * @param unSortedArray 未排序的数组
     * @return int[]  已经排序的数组
     */
    public static int[] doDescSort(int[] unSortedArray) {
        return doSort(unSortedArray, Sort.DESC);
    }

    /**
     * int 数组排序
     *
     * @param unSortedArray 待排序的数组
     * @param sortType      排序类型
     * @return 排序好的数组
     * @see Sort
     */
    public static int[] doSort(int[] unSortedArray, Sort sortType) {
        if (unSortedArray.length == 0) {
            return unSortedArray;
        }
        // 默认是 顺序排序
        boolean asc = true;
        if (Sort.DESC.toString().equalsIgnoreCase(sortType.toString())) {
            // 倒序排序
            asc = false;
        }
        // 带排序元素
        // 带排序元素前面的元素已经全部排序
        int currentValue;
        for (int i = 0; i < unSortedArray.length - 1; i++) {
            // 已被排序数据的索引
            int preIndex = i;
            currentValue = unSortedArray[preIndex + 1];
            //在已经排序好的元素面中寻找到当前元素合适的位置
            while (preIndex >= 0 && (asc ? currentValue < unSortedArray[preIndex] : currentValue > unSortedArray[preIndex])) {
                // 从已排序元素开始,将其与带排序元素逐一比较,以寻找带排序元素的合适位置
                unSortedArray[preIndex + 1] = unSortedArray[preIndex];
                preIndex--;
            }
            // while 循环结束时，说明已经找到了当前待排序数据的合适位置，插入
            unSortedArray[preIndex + 1] = currentValue;
        }
        return unSortedArray;
    }


}
