package cn.dankal.share.common;

import java.util.Arrays;

/**
 * 归并排序简单实现
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-02
 */
public class MergeSort {

    /**
     * 数组拆分默认临界值
     */
    public final static int DEFAULT_ARRAY_SPILT_THRESHOLD = 47;

    /**
     * 归并排序
     *
     * @param numArray 需要归并排序的数组
     * @return 排序好后的数组
     */
    public static int[] sort(int[] numArray) {
        if (numArray.length <= DEFAULT_ARRAY_SPILT_THRESHOLD) {
            return InsertionSort.doAscSort(numArray);
        } else {
            /*切分数组，然后递归调用*/
            int mid = numArray.length / 2;
            int[] left = Arrays.copyOfRange(numArray, 0, mid);
            int[] right = Arrays.copyOfRange(numArray, mid, numArray.length);
            return mergeSortedArray(sort(left), sort(right));
        }
    }

    /**
     * 将两个有序数组合并为一个有序数组(顺序排序)
     *
     * @param leftArray  left array
     * @param rightArray right array
     * @return 排好序后合并的数组
     */
    public static int[] mergeSortedArray(int[] leftArray, int[] rightArray) {
        int[] result = new int[leftArray.length + rightArray.length];
        for (int index = 0, i = 0, j = 0; index < result.length; index++) {
            if (i >= leftArray.length) {
                // 左边数组已经取完，完全取右边数组的值即可
                result[index] = rightArray[j++];
            } else if (j >= rightArray.length) {
                // 右边数组已经取完，完全取左边数组的值即可
                result[index] = leftArray[i++];
            } else if (leftArray[i] > rightArray[j]) {
                // 左边数组的元素值大于右边数组，取右边数组的值
                result[index] = rightArray[j++];
            } else {
                // 右边数组的元素值大于左边数组，取左边数组的值
                result[index] = leftArray[i++];
            }
        }
        return result;
    }
}
