package cn.dankal.share.concurrentutil;

import cn.dankal.share.common.ThreadExecutorFactory;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Semaphore;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * Semaphore 简单使用测试
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-02
 */
public class SemaphoreTest {

    public static void main(String[] args) {
        ThreadPoolExecutor threadPoolExecutor = ThreadExecutorFactory.getThreadPoolExecutor("semaphore-test");
        // semaphore: 令牌桶只有 3 个令牌
        Semaphore semaphore = new Semaphore(3);
        // 参与抢令牌的线程数
        final int count = 10;
        // 确保统计顺序执行完(成功/失败统计)
        CountDownLatch successAndFairCountDownLatch = new CountDownLatch(count);
        TokenBucket tokenBucket = new TokenBucket(semaphore, successAndFairCountDownLatch);
        try {
            for (int i = 0; i < count; i++) {
                threadPoolExecutor.execute(tokenBucket::getToken);
            }
            successAndFairCountDownLatch.await();
            System.out.println("参与线程数: " + count);
            System.out.println("成功线程数：" + tokenBucket.successCount());
            System.out.println("失败线程数：" + tokenBucket.fairCount());
        } catch (InterruptedException e) {
            // ignored
        } finally {
            threadPoolExecutor.shutdown();
        }
    }
}
