package cn.dankal.share.forkjoin;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ArrayUtil;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.RecursiveAction;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 基于 {@link RecursiveAction} (ForkJoin)实现 文件查找与简单统计
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-02
 */
public class FindFileTask extends RecursiveAction {

    /**
     * 目标文件夹
     */
    private final String targetFolder;

    /**
     * 目标文件后缀
     */
    private final String findFileSuffix;

    /**
     * 目标文件计数器
     */
    public final static AtomicInteger COUNTER_TARGET_FILE = new AtomicInteger(0);

    /**
     * 遍历目录计数器
     */
    public final static AtomicInteger COUNTER_EACH_FOLDER = new AtomicInteger(0);


    public FindFileTask(String targetFolder, String findFileSuffix) {
        this.targetFolder = targetFolder;
        this.findFileSuffix = findFileSuffix;
    }

    @Override
    protected void compute() {
        // 参数校验
        if (!FileUtil.exist(targetFolder) || FileUtil.isFile(targetFolder)) {
            throw new IllegalArgumentException("targetFolder 必须是存在的 文件夹");
        }
        File targetFile = FileUtil.file(targetFolder);
        File[] files = targetFile.listFiles();
        if (ArrayUtil.isEmpty(files)) {
            // 空文件夹直接返回
            return;
        }
        // 需要执行 Task List
        List<FindFileTask> subFindFileTask = new ArrayList<>(1 << 5);
        for (File file : files) {
            if (file.isDirectory()) {
                // 文件夹
                FindFileTask findFileTask = new FindFileTask(file.getAbsolutePath(), findFileSuffix);
                subFindFileTask.add(findFileTask);
                COUNTER_EACH_FOLDER.incrementAndGet();
            } else {
                // 文件
                String fileSuffix = FileUtil.getSuffix(file);
                if (("." + fileSuffix).equalsIgnoreCase(findFileSuffix)) {
                    // 是目标文件
                    System.out.println("find: " + file.getName());
                    COUNTER_TARGET_FILE.incrementAndGet();
                }
            }
        }
        if (CollectionUtil.isNotEmpty(subFindFileTask)) {
            // 执行所有拆分的 task 如果需要的话
            for (FindFileTask findFileTask : invokeAll(subFindFileTask)) {
                // 因为是拆分 即使我们不需要获取拆分结果也需要 join 切切
                findFileTask.join();
            }
        }
    }
}
