package cn.dankal.share.forkjoin;

import cn.dankal.share.common.IntArraySumUtil;

import java.util.concurrent.RecursiveTask;

/**
 * 基于 {@link RecursiveTask} 简单实现数组求和
 * <p>
 * 简单测试对比,展示 ForkJoin 的优势
 * </p>
 *
 * @author ZGH.MercyModest
 * @version V1.0.0
 * @create 2021-01-02
 */
public class IntArraySumTask extends RecursiveTask<Integer> {

    /**
     * 任务拆分临界值
     */
    private final int thresholdLength;

    /**
     * 求和目标数组
     */
    private final int[] intArray;

    /**
     * fromIndex
     */
    private final int fromIndex;

    /**
     * toIndex
     */
    private final int toIndex;

    public IntArraySumTask(int thresholdLength, int[] intArray, int fromIndex, int toIndex) {
        this.thresholdLength = thresholdLength;
        this.intArray = intArray;
        this.fromIndex = fromIndex;
        this.toIndex = toIndex;
    }

    @Override
    protected Integer compute() {
        if (toIndex - fromIndex <= thresholdLength) {
            return IntArraySumUtil.sumArray(intArray, fromIndex, toIndex);
        } else {
            //拆分成两个子 task 任务
            int middleIndex = (fromIndex + toIndex) / 2;
            IntArraySumTask leftSumTask = new IntArraySumTask(thresholdLength, intArray, fromIndex, middleIndex);
            IntArraySumTask rightSumTask = new IntArraySumTask(thresholdLength, intArray, middleIndex + 1, toIndex);
            invokeAll(leftSumTask, rightSumTask);
            // 归并拆分执行结果并返回
            return leftSumTask.join() + rightSumTask.join();
        }
    }
}
